import unittest
import dictns as namespace
import pickle
import json
from copy import deepcopy, copy

class Namespace(unittest.TestCase):

    def test_basic(self):
        n = namespace.Namespace({'a': {'b': {'c': 1}}})

        self.assertEqual(n.a.b.c, 1)
        self.assertEqual(n['a']['b']['c'], 1)
        self.assertEqual(n['a'].b['c'], 1)
        self.assertEqual(n['a']['b'].c, 1)
        self.assertEqual(n['a'].b.c, 1)
        self.assertEqual(list(n.keys()), ['a'])
        self.assertEqual(list(n.a.b.values()), [1])
        self.assertEqual(list(n.a.b.items()), [('c', 1)])
        n.a.b.c = 2
        self.assertEqual('a' in n, True)
        self.assertEqual('b' in n, False)

        self.assertEqual(n['a']['b']['c'], 2)
        n.a.b = {'d': 3}
        self.assertEqual(n.a.b.d, 3)
        n.a.b = namespace.Namespace({'e': 4})
        self.assertEqual(n.a.b.e, 4)
        self.assertRaises(KeyError, lambda: n.a.b.d == 3)
        self.assertEqual(namespace.Namespace(1), 1)
        self.assertEqual(namespace.Namespace([1]), [1])
        self.assertEqual(namespace.Namespace("1"), "1")
        self.assertEqual(namespace.Namespace(["1"]), ["1"])

        self.assertRaises(KeyError, lambda: n["__getitem__"])
        n.a['b'] = {'f': 5}
        self.assertEqual(n.a.b.f, 5)

    def test_nonzero(self):
        n = namespace.Namespace({'a': {'b': {'c': 1}}})
        self.failUnless(n)
        n = namespace.Namespace({})
        self.failIf(n)

    def test_list(self):
        n = namespace.Namespace([{'a': {'b': {
                                'c': 1}}}, {'a': {'b': {'c': 2}}}])
        self.assertEqual(n[0].a.b.c, 1)
        self.assertEqual(n[1].a.b.c, 2)
        for i in n:
            self.assertEqual(i.a.b.c, i.a.b.c)

    def test_jsondump(self):
        s = '[{"a": {"b": {"c": 1}}}, {"a": {"b": {"c": 2}}}]'
        n = namespace.Namespace(json.loads(s))
        self.assertEqual(json.dumps(n), s)

    def test_prettyprint(self):
        n = namespace.Namespace({'a': [{'b': {'c': 1}}]})
        expected = """\
{
    "a": [
        {
            "b": {
                "c": 1
            }
        }
    ]
}"""
        self.assertEqual(repr(n), expected)
        expected = """\
a -> list
a[i] -> dict
a[i].b -> dict
a[i].b.c -> int
"""
        self.assertEqual(namespace.documentNamespace(n), expected)

    def test_pickle(self):
        n = namespace.Namespace([{'a': {'b': {
                                'c': 1}}}, {'a': {'b': {'c': 2}}}])
        s = pickle.dumps(n)
        n = pickle.loads(s)
        self.assertEqual(n[0].a.b.c, 1)
        self.assertEqual(n[1].a.b.c, 2)
        for i in n:
            self.assertEqual(i.a.b.c, i.a.b.c)

    def test_deepcopy(self):
        a = namespace.Namespace(
            {'a': dict(b=[1, 2, 3])})

        b = deepcopy(a)
        self.assertEqual(a, b)
        self.assertNotEqual(id(a), id(b))
        self.assertNotEqual(id(a.a.b), id(b.a.b))

    def test_copy(self):
        a = namespace.Namespace(
            {'a': dict(b=[1, 2, 3])})

        b = copy(a)
        self.assertEqual(a, b)
        self.assertNotEqual(id(a), id(b))
        self.assertEqual(id(a.a.b), id(b.a.b))



import json
from copy import deepcopy, copy


class Namespace(dict):
    """
    A convenience class that makes a json like dict of (dicts,lists,strings,integers)
    looks like a python object allowing syntax sugar like mydict.key1.key2.key4 = 5
    this object also looks like a dict so that you can walk items(), keys() or values()

    input should always be json-able but this is checked only in pedentic mode, for
    obvious performance reason
    """
    __slots__ = []

    def __init__(self, d):
        dict.__init__(self, d)
        for k, v in d.items():
            # if already a Namespace, this will not match
            if type(v) in (dict, list):
                dict.__setitem__(self, k, Namespace(v))

    def __new__(cls, v):
        if type(v) == dict:
            return dict.__new__(cls, v)
        elif type(v) == list:
            return [Namespace(i) for i in v]
        else:
            return v
# pretty printing

    def __repr__(self):
        """ pretty printed __repr__, for debugging"""
        d = dict(self)
        try:
            return json.dumps(d, indent=4)
        except Exception:
            return repr(d)

# on the fly conversion
    def __setitem__(self, k, v):
        dict.__setitem__(self, k, Namespace(v))

# object like accessors
    __getattr__ = dict.__getitem__
    __setattr__ = __setitem__

    def __getstate__(self):
        return self

    def __setstate__(self, _dict):
        self.__init__(_dict)

# deepcopy
    def __deepcopy__(self, memo):
        return Namespace(deepcopy(dict(self)))

# copy
    def __copy__(self):
        return Namespace(dict(self))


def documentNamespace(n, parent=None):
    """This prints the available keys and subkeys of the data, and their types,
    meant for quick auto-documentation of big json data
    """
    s = ""
    for k, v in n.items():
        if parent:
            me = parent + "." + k
        else:
            me = k

        def do_item(me, v):
            t = type(v).__name__
            if t == "Namespace":
                t = "dict"
            s = me + " -> " + t + "\n"
            if isinstance(v, dict):
                v = Namespace(v)
                s += documentNamespace(v, me)
            elif type(v) == list:
                if len(v) > 0:
                    v = v[0]
                    s += do_item(me + "[i]", v)
            return s
        s += do_item(me, v)
    return s



#!/usr/bin/env python
"""Installs dictns using setuptools

Run:
    python setup.py install
to install the package from the source archive.
"""

from setuptools import setup, find_packages

version = 1.0

if __name__ == "__main__":
    extraArguments = {
        'classifiers': [
            """License :: OSI Approved :: BSD License""",
            """Programming Language :: Python""",
            """Topic :: Software Development :: Libraries :: Python Modules""",
            """Intended Audience :: Developers""",
        ],
        'keywords': 'dict, object',
        'long_description': """simple class that merges dictionary and object API

This Namespace objects work in a similar way as javascript objects.
usage:
    from dictns import Namespace
    n = Namespace(dict(a=1, b=3, c=dict(d=4)))
    assert(n['a'] == n.a)
    assert(n['c']['c] == n.c.d)
""",
        'platforms': ['Any'],
    }
    ### Now the actual set up call
    setup(
        name="dictns",
        version=version,
        url="http://github.com/tardyp/dictns",
        download_url="http://github.com/tardyp/dictns",
        description="simple class that merge dictionary and object API",
        author="Pierre Tardy",
        author_email="tardyp@gmail.com",
        install_requires=[
        ],
        license="BSD",
        py_modules=[
            'dictns',
        ],
        **extraArguments
    )
